' Licensed Materials - Property of IBM
'
' (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
'
' US Government Users Restricted Rights - Use, duplicate or
' disclosure restricted by GSA ADP Schedule Contract with
' IBM Corp.

'==============================
' Initialize
'==============================
  Set wbemSvc = GetObject("winmgmts://" & "." & "/root/cimv2")
  Set fileSys = CreateObject("Scripting.FileSystemObject")
  osCaption = GetOSCaption

'==============================
' Main(Args[])
'==============================

  PrintMetaInfo
  EnumerateChassis
  EnumerateProcessors
  EnumeratePhysicalMemory
  EnumerateDiskDrives
'  EnumerateLogicalVolumes
  EnumerateFileSystems
  EnumerateFibreChannel
  EnumerateDNSSettings
  EnumerateNICs
  EnumerateLocation
  EnumerateUserContact
  
'==============================
' EnumerateChassis
'==============================
Function EnumerateChassis()

  On Error Resume Next
  Set csSet = wbemSvc.ExecQuery("Select * from Win32_ComputerSystemProduct")

  index = 0
  For Each csObj In csSet
    uuid = FormatUUID(csObj.UUID)
    model = ParseModel(csObj.Name)
    machType = ParseMachineType(csObj.Name)
    name = csObj.Vendor & "," & machType & "-" & model & "-" & csObj.IdentifyingNumber
'    changed for defect 165353
'    tag = "U" & machType & "." & model & "." & csObj.IdentifyingNumber
    tag = csObj.Vendor & " " & machType & model & " " & csObj.IdentifyingNumber

    PrintProperty "Chassis",index,"Name",name
    PrintProperty "Chassis",index,"Tag",tag
    PrintProperty "Chassis",index,"Manufacturer",csObj.Vendor
    PrintProperty "Chassis",index,"Model",model
    PrintProperty "Chassis",index,"VendorType",machType
    PrintProperty "Chassis",index,"SerialNumber",csObj.IdentifyingNumber
    PrintProperty "Chassis",index,"PackageType","3"
    PrintProperty "Chassis",index,"PackageGUID",uuid
    index = index + 1
  Next

End Function

'==============================
' EnumerateProcessors
'==============================
Function EnumerateProcessors()

  On Error Resume Next
  Set procSet = wbemSvc.ExecQuery("Select * from Win32_Processor")

  index = 0
  For Each proc In procSet
    PrintProperty "Processor",index,"Name",proc.Name
    PrintProperty "Processor",index,"DeviceID",proc.DeviceID
    PrintProperty "Processor",index,"AddressWidth",proc.AddressWidth
    PrintProperty "Processor",index,"DataWidth",proc.DataWidth
    PrintProperty "Processor",index,"CurrentClockSpeed",proc.CurrentClockSpeed
    PrintProperty "Processor",index,"MaxClockSpeed",proc.MaxClockSpeed
    PrintProperty "Processor",index,"Family",proc.Family
    PrintProperty "Processor",index,"UniqueID",proc.ProcessorId
    PrintProperty "Processor",index,"NumberOfEnabledCores",proc.NumberOfCores
    PrintStateProperty "Processor",index,"HealthState",proc.HealthState
    index = index + 1
  Next

End Function

'==============================
' EnumeratePhysicalMemory
'==============================
Function EnumeratePhysicalMemory()

  On Error Resume Next
  Set csSet = wbemSvc.ExecQuery("Select * from Win32_ComputerSystemProduct")

  For Each csObj In csSet
    model = ParseModel(csObj.Name)
    machType = ParseMachineType(csObj.Name)
'    changed for defect 165353
'    tag = "U" & machType & "." & model & "." & csObj.IdentifyingNumber
    tag = csObj.Vendor & " " & machType & model & " " & csObj.IdentifyingNumber
  Next

  Set memSet = wbemSvc.ExecQuery("Select * from Win32_PhysicalMemory")

  index = 0
  For Each mem In memSet
    PrintProperty "PhysicalMemory",index,"Name",mem.DeviceLocator
    PrintProperty "PhysicalMemory",index,"Tag",tag & "-P1-C" & (index + 1)
    PrintProperty "PhysicalMemory",index,"Manufacturer",mem.Manufacturer
    PrintProperty "PhysicalMemory",index,"Model",mem.Model
    PrintProperty "PhysicalMemory",index,"SerialNumber",mem.SerialNumber
    PrintProperty "PhysicalMemory",index,"PartNumber",mem.PartNumber
    PrintProperty "PhysicalMemory",index,"FormFactor",mem.FormFactor
    PrintProperty "PhysicalMemory",index,"BankLabel",mem.BankLabel
    PrintProperty "PhysicalMemory",index,"Capacity",mem.Capacity
    PrintProperty "PhysicalMemory",index,"DataWidth",mem.DataWidth
    PrintProperty "PhysicalMemory",index,"InterleavePosition",mem.InterleavePosition
    PrintProperty "PhysicalMemory",index,"MemoryType",mem.MemoryType
    PrintProperty "PhysicalMemory",index,"PositionInRow",mem.PositionInRow
    PrintProperty "PhysicalMemory",index,"Speed",mem.Speed
    PrintProperty "PhysicalMemory",index,"TotalWidth",mem.TotalWidth
    index = index + 1
  Next

End Function

'==============================
' EnumerateDiskDrives
'==============================
Function EnumerateDiskDrives()

  On Error Resume Next
  Set driveSet = wbemSvc.ExecQuery("Select * from Win32_DiskDrive")

  index = 0
  For Each drive In driveSet
    If InStr(drive.Caption,"LSI") = 0 AND InStr(drive,drive.Caption,"Logical") = 0 Then
      PrintProperty "DiskDrive",index,"Name",drive.Caption
      PrintProperty "DiskDrive",index,"DeviceID",drive.DeviceID
      PrintProperty "DiskDrive",index,"DefaultBlockSize",drive.DefaultBlockSize
      PrintProperty "DiskDrive",index,"MaxBlockSize",drive.MaxBlockSize
      PrintProperty "DiskDrive",index,"MaxMediaSize",drive.MaxMediaSize
      PrintProperty "DiskDrive",index,"MinBlockSize",drive.MinBlockSize
      PrintProperty "DiskDrive",index,"NeedsCleaning",drive.NeedsCleaning
      PrintProperty "DiskDrive",index,"Capacity",drive.Size
      PrintStateProperty "DiskDrive",index,"HealthState",proc.HealthState
      index = index + 1
    End If
  Next

End Function

'==============================
' EnumerateLogicalVolumes
'==============================
Function EnumerateLogicalVolumes()

  On Error Resume Next
  Set volumeSet = wbemSvc.ExecQuery("Select * from Win32_LogicalDisk where (DriveType=3 OR DriveType=5)")

  index = 0
  For Each volume In volumeSet
    PrintProperty "LogicalVolume",index,"Name",volume.Name
    PrintProperty "LogicalVolume",index,"DeviceID",volume.DeviceID
    PrintProperty "LogicalVolume",index,"BlockSize","1"
    PrintProperty "LogicalVolume",index,"ConsumableBlocks",volume.Size
    PrintProperty "LogicalVolume",index,"Capacity",volume.Size
    PrintProperty "LogicalVolume",index,"FreeSpace",volume.FreeSpace
    PrintProperty "LogicalVolume",index,"DataOrganization",volume.DataOrganization
    PrintProperty "LogicalVolume",index,"DataRedundancy",volume.DataRedundancy
    PrintProperty "LogicalVolume",index,"DeltaReservation",volume.DeltaReservation
    PrintProperty "LogicalVolume",index,"NumberOfBlocks",volume.Size
    PrintProperty "LogicalVolume",index,"PackageRedundancy",volume.PackageRedundancy
    PrintProperty "LogicalVolume",index,"SequentialAccess"",volume.SequentialAccess"
    index = index + 1
  Next

End Function

'==============================
' EnumerateFileSystems
'==============================
Function EnumerateFileSystems()

  On Error Resume Next
  Set Drives = fileSys.Drives

  index = 0
  For Each drive In Drives
    If drive.IsReady Then
      If Not drive.DriveType = 3 Then
        PrintProperty "FileSystem",index,"Name",drive.Path
        PrintProperty "FileSystem",index,"Root",Replace(drive.RootFolder, "\", "\\")
        PrintProperty "FileSystem",index,"HealthState","1"
        PrintProperty "FileSystem",index,"AvailableSpace",drive.AvailableSpace
        PrintProperty "FileSystem",index,"FileSystemSize",drive.TotalSize
        fsType="0"
        If StrComp(drive.FileSystem,"FAT",1) = 0 Then fsType="9"
        If StrComp(drive.FileSystem,"NFS",1) = 0 Then fsType="25"
        If StrComp(drive.FileSystem,"UDF",1) = 0 Then fsType="16"
        If StrComp(drive.FileSystem,"NTFS",1) = 0 Then fsType="6"
        If StrComp(drive.FileSystem,"EXT2",1) = 0 Then fsType="12"
        If StrComp(drive.FileSystem,"EXT3",1) = 0 Then fsType="13"
        If StrComp(drive.FileSystem,"FAT16",1) = 0 Then fsType="8"
        If StrComp(drive.FileSystem,"FAT32",1) = 0 Then fsType="9"
        PrintProperty "FileSystem",index,"FileSystemType",fsType
        label="None"
        If Len(drive.ShareName) > 0 Then label=drive.ShareName
        If Len(drive.VolumeName) > 0 Then label=drive.VolumeName
        PrintProperty "FileSystem",index,"Label",label
        index = index + 1
      End If
    End If
  Next

End Function

'==============================
' EnumerateFibreChannel
'==============================
Function EnumerateFibreChannel()
  origName = "hbaapi.dll"
  bakName = "hbaapi.dll.bak"
  If fileSys.FileExists(origName) = true AND InStr(osCaption,"2008") > 0 Then
    Set f2 = fileSys.GetFile(origName)
    f2.Move (bakName)
  End If
  RunFCCommand
  If fileSys.FileExists(bakName) = true AND InStr(osCaption,"2008") > 0 Then
    Set f2 = fileSys.GetFile(bakName)
    f2.Move (origName)
  End If
End Function

'==============================
' PrintHBA
'==============================
Function PrintHBA(index,deviceId,portWWN,nodeWWN,vendor,model,serial,driverVer,firmVer,fabric,state,speed,biosVer,name)
  If Len(deviceId) > 0 And Len(portWWN) > 0 And InStr(vendor,"Brocade") = 0 Then
 
    PrintProperty "HBAAdapter",index,"Name",portWWN
    PrintProperty "FCPort",index,"Name",portWWN
    PrintProperty "FCPort",index,"DeviceID",portWWN
    PrintProperty "FCPort",index,"Model",model
    PrintProperty "FCPort",index,"LinkTechnology","4"
    PrintProperty "FCPort",index,"PermanentAddress",portWWN
    PrintProperty "FCPort",index,"FabricName",fabric
    PrintProperty "FCPort",index,"OperatingState",state
    PrintProperty "FCPort",index,"Speed",speed
    PrintProperty "SCSIPort",index,"Name",portWWN
    PrintProperty "SCSIPort",index,"ConnectionType",1
    PrintProperty "SCSIPort",index,"Role",1
    PrintProperty "SCSIPort",index,"OperatingState",state
    PrintProperty "HBACard",index,"Name",model
    If Len(serial) > 0 Then
       PrintProperty "HBACard",index,"Tag",serial
       PrintProperty "HBACard",index,"SerialNumber",serial
       PrintProperty "HBAAdapter",index,"DeviceID",serial
    Else
       PrintProperty "HBACard",index,"Tag",deviceId
       PrintProperty "HBAAdapter",index,"DeviceID",deviceId
    End If
    PrintProperty "HBACard",index,"Model",model
    PrintProperty "HBACard",index,"Manufacturer",vendor
    PrintProperty "HBASoftware",index*3,"Name",name & " HBA Firmware"
    PrintProperty "HBASoftware",index*3,"Vendor",vendor
    PrintProperty "HBASoftware",index*3,"Version",firmVer
    PrintProperty "HBASoftware",index*3,"Category","7"
    PrintProperty "HBASoftware",index*3,"SoftwareID",model
    PrintProperty "HBASoftware",index*3,"SubCategory","HBA"
    PrintProperty "HBASoftware",index*3,"InstalledLocation","System0"
    PrintProperty "HBASoftware",index*3+1,"Name",name & " HBA Driver"
    PrintProperty "HBASoftware",index*3+1,"Vendor",vendor
    PrintProperty "HBASoftware",index*3+1,"Version",driverVer
    PrintProperty "HBASoftware",index*3+1,"Category","9"
    PrintProperty "HBASoftware",index*3+1,"SoftwareID",model
    PrintProperty "HBASoftware",index*3+1,"SubCategory","HBA"
    PrintProperty "HBASoftware",index*3+1,"InstalledLocation","System0"
    PrintProperty "HBASoftware",index*3+2,"Name",name & " HBA BIOS"
    PrintProperty "HBASoftware",index*3+2,"Vendor",vendor
    PrintProperty "HBASoftware",index*3+2,"Version",biosVer
    PrintProperty "HBASoftware",index*3+2,"Category","8"
    PrintProperty "HBASoftware",index*3+2,"SoftwareID",model
    PrintProperty "HBASoftware",index*3+2,"SubCategory","HBA"
    PrintProperty "HBASoftware",index*3+2,"InstalledLocation","System0"
    PrintProperty "HBACardRealizes",index,"SourceId",index
    PrintProperty "HBACardRealizes",index,"TargetId",index
    PrintProperty "HBAControlledBy",index,"SourceId",index
    PrintProperty "HBAControlledBy",index,"TargetId",index
    PrintProperty "HBAAccessedVia",index,"SourceId",index
    PrintProperty "HBAAccessedVia",index,"TargetId",index
  End If
End Function

'==============================
' EnumerateDNSSettings
'==============================
Function EnumerateDNSSettings()

  On Error Resume Next
  Set dnsSet = wbemSvc.ExecQuery("Select * from Win32_NetworkAdapterConfiguration")

  index = 0
  For Each dns In dnsSet
    If dns.FullDNSRegistrationEnabled = "True" AND IsNull(dns.IPAddress) = "False" Then
     If InStr(dns.IPAddress(0),"0.0.0.0") = 0 Then
      PrintProperty "DNSSettings",index,"Name",dns.Description
      PrintProperty "DNSSettings",index,"SettingID",dns.SettingID
      PrintProperty "DNSSettings",index,"DomainName",dns.DNSDomain
      PrintProperty "DNSSettings",index,"RequestedHostname",dns.DNSHostName
      PrintProperty "DNSSettings",index,"RegisterThisConnectionsAddress",dns.FullDNSRegistrationEnabled
      PrintProperty "DNSSettings",index,"UseSuffixWhenRegistering",dns.DomainDNSRegistrationEnabled
      PrintArrayProperty "DNSSettings",index,"DNSServerAddresses",dns.DNSServerSearchOrder
      PrintArrayProperty "DNSSettings",index,"DNSDomainSuffixes",dns.DNSDomainSuffixSearchOrder
      index = index + 1
     End If
    End If
  Next

End Function

'==============================
' EnumerateNICs
'==============================
Function EnumerateNICs()

  On Error Resume Next
  Set dnsSet = wbemSvc.ExecQuery("Select * from Win32_NetworkAdapterConfiguration")

  index = 0
  lanindex = 0
  For Each dns In dnsSet
    If IsNull(dns.IPAddress) = "False" Then
     If InStr(dns.IPAddress(0),"0.0.0.0") = 0 Then
      Set adpSet = wbemSvc.ExecQuery("Select * from Win32_NetworkAdapter where Index = " & dns.Index)
      intfName = Replace(dns.MACAddress,":","")
      For Each adp In adpSet
        intfName = adp.NetConnectionID
      Next
      
      ipindex = 0
      For Each x in dns.IPAddress
        ipPropName = "IPv4Address"
        maskPropName = "SubnetMask"
        If (InStr(dns.IPAddress(ipindex),".") = 0) Then 
          ipPropName = "IPv6Address"
          maskPropName = "PrefixLength"
        End If

        PrintProperty "CSIPProtocolEndpoint",index,"Name",dns.IPAddress(ipindex)
        PrintNonArrayProperty "CSIPProtocolEndpoint",index,"Gateway",dns.DefaultIPGateway
        PrintProperty "CSIPProtocolEndpoint",index,maskPropName,dns.IPSubnet(ipindex)
        PrintProperty "CSIPProtocolEndpoint",index,ipPropName,dns.IPAddress(ipindex)
        PrintProperty "CSIPProtocolEndpoint",index,"HostType","CS"

        PrintProperty "OSIPProtocolEndpoint",index,"Name",dns.IPAddress(ipindex)
        PrintNonArrayProperty "OSIPProtocolEndpoint",index,"Gateway",dns.DefaultIPGateway
        PrintProperty "OSIPProtocolEndpoint",index,maskPropName,dns.IPSubnet(ipindex)
        PrintProperty "OSIPProtocolEndpoint",index,ipPropName,dns.IPAddress(ipindex)
        PrintProperty "OSIPProtocolEndpoint",index,"HostType","OS"

        PrintProperty "CSBindsTo",index,"SourceId",index
        PrintProperty "CSBindsTo",index,"TargetId",lanindex
        PrintProperty "OSBindsTo",index,"SourceId",index
        PrintProperty "OSBindsTo",index,"TargetId",lanindex
        index = index + 1
        ipindex = ipindex + 1
      Next
   
      tempMac = Replace(dns.MACAddress,":","")
      p = InStr(tempMac," ")
      tempMac = Mid(tempMac, 1, p-1)

      PrintProperty "CSLANEndpoint",lanindex,"Name",tempMac
      PrintProperty "CSLANEndpoint",lanindex,"DisplayName",intfName
      PrintProperty "CSLANEndpoint",lanindex,"MACAddress",tempMac
      PrintProperty "CSLANEndpoint",lanindex,"DeviceName",LookupIntfName(dns.Description)
      PrintProperty "CSLANEndpoint",lanindex,"HostType","CS"
      PrintProperty "OSLANEndpoint",lanindex,"Name",tempMac
      PrintProperty "OSLANEndpoint",lanindex,"DisplayName",intfName
      PrintProperty "OSLANEndpoint",lanindex,"MACAddress",tempMac
      PrintProperty "OSLANEndpoint",lanindex,"DeviceName",LookupIntfName(dns.Description)
      PrintProperty "OSLANEndpoint",lanindex,"HostType","OS"
      lanindex = lanindex + 1
     End If
    End If
  Next

End Function

'==============================
' EnumerateLocation
'==============================
Function EnumerateLocation()

  isFirst = true
  directorRoot = GetDirectorRoot
  userIniFile = directorRoot & "\data\twguser.ini"

  On Error Resume Next

  If Len(directorRoot) > 0 And fileSys.FileExists(userIniFile) = true Then

    Set txtStream = fileSys.OpenTextFile(userIniFile, 1, false)
    Do While Not txtStream.AtEndOfStream
      line = Trim(txtStream.ReadLine)

      If Not InStr(line,"#") = 1 Then
        sepIdx = InStr(line,"=")
        lastIdx = Len(line) - 1

        If sepIdx > 1 And sepIdx < lastIdx Then
          propName = Trim(Mid(line,1,sepIdx-1))
          propVal = Trim(Mid(line,sepIdx+1))

          If isFirst = true Then
            PrintProperty "Location",0,"Name","<Chassis.0.PackageGUID>:DefaultLocation"
            PrintProperty "Location",0,"DisplayName","Default Location"
            isFirst = false
          End If

          If StrComp(propName,"CITY",1) = 0 Then PrintProperty "Location",0,"City",propVal
          If StrComp(propName,"STATE",1) = 0 Then PrintProperty "Location",0,"StateOrProvince",propVal
          If StrComp(propName,"ZIPCODE",1) = 0 Then PrintProperty "Location",0,"ZipCode",propVal
          If StrComp(propName,"COUNTRY",1) = 0 Then PrintProperty "Location",0,"Country",propVal
          If StrComp(propName,"BUILDING",1) = 0 Then PrintProperty "Location",0,"Building",propVal
          If StrComp(propName,"FLOOR",1) = 0 Then PrintProperty "Location",0,"Floor",propVal
          If StrComp(propName,"ROOM_NUMBER",1) = 0 Then PrintProperty "Location",0,"RoomNumber",propVal
          If StrComp(propName,"LATITUDE",1) = 0 Then PrintProperty "Location",0,"Latitude",propVal
          If StrComp(propName,"LONGITUDE",1) = 0 Then PrintProperty "Location",0,"Longitude",propVal
        End If
      End If
    Loop
    txtStream.close

  End If

End Function

'==============================
' EnumerateUserContact
'==============================
Function EnumerateUserContact()

  firstName = ""
  lastName = ""
  middleName = ""
  directorRoot = GetDirectorRoot
  userIniFile = directorRoot & "\data\twguser.ini"

  On Error Resume Next

  If Len(directorRoot) > 0 And fileSys.FileExists(userIniFile) = true Then

    Set txtStream = fileSys.OpenTextFile(userIniFile, 1, false)
    Do While Not txtStream.AtEndOfStream
      line = Trim(txtStream.ReadLine)

      If Not InStr(line,"#") = 1 Then
        sepIdx = InStr(line,"=")
        lastIdx = Len(line) - 1

        If sepIdx > 1 And sepIdx < lastIdx Then
          propName = Trim(Mid(line,1,sepIdx-1))
          propVal = Trim(Mid(line,sepIdx+1))

          If StrComp(propName,"FIRST_NAME",1) = 0 Then firstName = propVal
          If StrComp(propName,"MIDDLE_NAME",1) = 0 Then middleName = propVal
          If StrComp(propName,"LAST_NAME",1) = 0 Then lastName = propVal
          If StrComp(propName,"EMPLOYEE_ID",1) = 0 Then PrintProperty "UserContact",0,"EmployeeNumber",propVal
          If StrComp(propName,"TITLE",1) = 0 Then PrintProperty "UserContact",0,"Title",propVal
          If StrComp(propName,"DEPT_NAME",1) = 0 Then PrintProperty "UserContact",0,"Department",propVal
          If StrComp(propName,"DEPT_NUMBER",1) = 0 Then PrintProperty "UserContact",0,"DeptNumber",propVal
          If StrComp(propName,"DIVISION",1) = 0 Then PrintProperty "UserContact",0,"Division",propVal
          If StrComp(propName,"WORK_PHONE",1) = 0 Then PrintProperty "UserContact",0,"TelephoneNumber",propVal
          If StrComp(propName,"CELLULAR_PHONE",1) = 0 Then PrintProperty "UserContact",0,"Mobile",propVal
          If StrComp(propName,"FAX_NUMBER",1) = 0 Then PrintProperty "UserContact",0,"FaxNumber",propVal
          If StrComp(propName,"PAGER_NUMBER",1) = 0 Then PrintProperty "UserContact",0,"Pager",propVal
          If StrComp(propName,"PAGER_TYPE",1) = 0 Then PrintProperty "UserContact",0,"PagerType",propVal
          If StrComp(propName,"EMAIL_ADDRESS",1) = 0 Then PrintProperty "UserContact",0,"EMail",propVal
          If StrComp(propName,"HOME_PHONE",1) = 0 Then PrintProperty "UserContact",0,"HomePhone",propVal
        End If
      End If
    Loop
    txtStream.close

    fullName = ParseFullName(firstName,middleName,lastName)
    If Len(fullName) > 0 Then
      PrintProperty "UserContact",0,"Name",fullName
      PrintProperty "UserContact",0,"FullName",fullName
    End If
  End If

End Function

'==============================
' RunFCCommand
'==============================
Function RunFCCommand()
  Dim deviceId, vendor, model, serial, state, speed
  Dim driverVer, firmVer, fabric, portWWN, nodeWWN
  Set WshShell = CreateObject("WScript.Shell")
  On Error Resume Next
  Set WshExec = WshShell.Exec("fcinfo.exe")

  count=""
  If Err.Number = 0 Then
    Do While Not WshExec.StdOut.AtEndOfStream
      line = WshExec.StdOut.ReadLine

      If Len(line) > 0 And InStr(line,"FCPort.") = 1 Then
        dotIdx = InStr(8,line,".")
        asgIdx = InStr(8,line,"=")

        If asgIdx < Len(line) And dotIdx > 1 And asgIdx > dotIdx Then
          newCount = Mid(line,8,dotIdx-8)
          propName = Mid(line,dotIdx+1,asgIdx-dotIdx-1)
          propVal = Mid(line,asgIdx+1)

          If Not StrComp(count, newCount) = 0 Then

	    If InStr(vendor,"QLogic") > 0 Then
		scliAllFirmware portWWN,firmVer,biosVer,serial
		name = "QLogic"
    	    Else
		name = vendor
    	    End If

            PrintHBA count,deviceId,portWWN,nodeWWN,vendor,model,serial,driverVer,firmVer,fabric,state,speed,biosVer,name
            deviceId = vendor = model = serial = state = speed = name = ""
            driverVer = firmVer = fabric = portWWN = nodeWWN = biosver = ""
            count = newCount
          End If

          If StrComp(propName,"DeviceID",1) = 0 Then deviceId = propVal
          If StrComp(propName,"PortWWN",1) = 0 Then portWWN = propVal
          If StrComp(propName,"NodeWWN",1) = 0 Then nodeWWN = propVal
          If StrComp(propName,"Vendor",1) = 0 Then vendor = propVal
          If StrComp(propName,"Model",1) = 0 Then model = propVal
          If StrComp(propName,"SerialNumber",1) = 0 Then serial = propVal
          If StrComp(propName,"DriverVersion",1) = 0 Then driverVer = propVal
          If StrComp(propName,"FirmwareVersion",1) = 0 Then firmVer = propVal
          If StrComp(propName,"FabricName",1) = 0 Then fabric = propVal
          If StrComp(propName,"OperatingState",1) = 0 Then state = propVal
          If StrComp(propName,"Speed",1) = 0 Then speed = propVal
        End If
      End If
    Loop

    If InStr(vendor,"QLogic") > 0 Then
	scliAllFirmware portWWN,firmVer,biosVer,serial
	name = "QLogic"
    Else
	name = vendor
    End If

    PrintHBA count,deviceId,portWWN,nodeWWN,vendor,model,serial,driverVer,firmVer,fabric,state,speed,biosver,name
  End If
    
End Function

'==============================
' LookupIntfName
'==============================
Function LookupIntfName(name)

  intfName = ""
  fmtName = name
  On Error Resume Next

  fmtName = Replace(fmtName, "/", "_")
  fmtName = Replace(fmtName, "#", "_")
  fmtName = Replace(fmtName, "(", "%")
  fmtName = Replace(fmtName, ")", "%")

  Set tcpSet1 = wbemSvc.ExecQuery("Select * from Win32_PerfFormattedData_Tcpip_NetworkInterface where Name like '" & fmtName & "'")
  For Each tcp1 In tcpSet1
    intfName = tcp1.Name
  Next
  
  If Len(intfName) = 0 Then
    Set tcpSet2 = wbemSvc.ExecQuery("Select * from Win32_PerfFormattedData_Tcpip_NetworkInterface where Name like '" & fmtName & " - %'")
    For Each tcp2 In tcpSet2
      intfName = tcp2.Name
    Next
  End If  
  If Len(intfName) = 0 Then intfName = name

  LookupIntfName = intfName

End Function

'==============================
' GetOSCaption
'==============================
Function GetOSCaption()

  GetOSCaption= "WINDOWS"
  Set osSet = wbemSvc.ExecQuery("Select * from Win32_OperatingSystem")

  For Each osInst In osSet
    GetOSCaption = osInst.Caption
  Next

End Function

'==============================
' GetDirectorRoot
'==============================
Function GetDirectorRoot()

  GetDirectorRoot= ""
  const HKEY_LOCAL_MACHINE = &H80000002
  strKeyPath = "SOFTWARE\IBM\IBM Director Agent\CurrentVersion"
  strKeyName = "Root"

  On Error Resume Next
  Set objReg = GetObject("winmgmts:\\" & "." & "\root\default:StdRegProv")
  objReg.GetStringValue HKEY_LOCAL_MACHINE,strKeyPath,strKeyName,strKeyValue

  If IsNull(strKeyValue) = false Then GetDirectorRoot = strKeyValue

End Function

'==============================
' FormatUUID
'==============================
Function FormatUUID(uuid)

    FormatUUID = Replace(uuid, "-", "")
    If Len(FormatUUID) = 32 Then
      TempUUID1 = Mid(FormatUUID,7,2) & Mid(FormatUUID,5,2) & Mid(FormatUUID,3,2) & Mid(FormatUUID,1,2)
      TempUUID2 = Mid(FormatUUID,11,2) & Mid(FormatUUID,9,2) & "-" & Mid(FormatUUID,15,2) & Mid(FormatUUID,13,2)
      FormatUUID = TempUUID1 & "-" & TempUUID2 & "-" & Mid(FormatUUID, 17, 4) & "-" & Mid(FormatUUID, 21, 12)
    End If

End Function

'==============================
' FormatProductId
'==============================
Function FormatProductId(productId)

    FormatProductId = productId
    If InStr(FormatProductId,"[") > 0 Then
      FormatProductId = Mid(FormatProductId, InStr(FormatProductId,"[")+1)
    End If
    If InStr(FormatProductId,"]") > 0 Then
      FormatProductId = Mid(FormatProductId, 1, InStr(FormatProductId,"]")-1)
    End If

End Function

'==============================
' ParseModel
'==============================
Function ParseModel(productId)

    ParseModel = FormatProductId(productId)
    If Len(ParseModel) = 7 Then
      ParseModel = Mid(ParseModel, 5)
    End If

End Function

'==============================
' ParseMachineType
'==============================
Function ParseMachineType(productId)

    model = FormatProductId(productId)
    ParseMachineType = ""
    If Len(model) = 7 Then
      ParseMachineType = Mid(model, 1, 4)
    End If

End Function

'==============================
' ParseFullName
'==============================
Function ParseFullName(firstName, middleName, lastName)

    ParseFullName = ""
    If Len(firstName) > 0 Then ParseFullName = ParseFullName & firstName & " "
    If Len(middleName) > 0 Then ParseFullName = ParseFullName & middleName & " "
    If Len(lastName) > 0 Then ParseFullName = ParseFullName & lastName & " "
    ParseFullName = Trim(ParseFullName)

End Function

'==============================
' PrintProperty
'==============================
Function PrintProperty(rscType, index, propName, propVal)

  If IsNull(propVal) = "False" AND Len(propVal) > 0 Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=" & propVal
  End If

End Function

'==============================
' PrintArrayProperty
'==============================
Function PrintArrayProperty(rscType, index, propName, propVal)

  index2 = 0
  If IsNull(propVal) = "False" AND IsArray(propVal) = "True" Then
    For Each x in propVal
      If index2 = 0 Then
        WScript.StdOut.Write rscType & "." & index & "." & propName & "=" & x
      Else
        WScript.StdOut.Write ";" & x
      End If
      index2 = index2 + 1
    Next
    If index2 > 0 Then
      Wscript.StdOut.WriteLine ""
    End If
  End If

End Function

'==============================
' PrintNonArrayProperty
'==============================
Function PrintNonArrayProperty(rscType, index, propName, propVal)

  index2 = 0
  If IsNull(propVal) = "False" AND IsArray(propVal) = "True" Then
    For Each x in propVal
      If index2 = 0 Then
        WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=" & x
      End If
      index2 = index2 + 1
    Next
  End If

End Function

'==============================
' PrintStateProperty
'==============================
Function PrintStateProperty(rscType, index, propName, propVal)

  If propVal = "OK" Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=1"
  End If
  If propVal = "Error" Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=3"
  End If

End Function

'==============================
' scliAllFirmware
'==============================

Function scliAllFirmware(portWWN,firmVer,biosVer,serialNum)

  Dim colonIdx 
  Dim snDone,ffvDone,fbvDone
  Set WshScliShell = CreateObject("WScript.Shell")
  On Error Resume Next
  Set WshScliExec = WshScliShell.Exec("scli -z " & portWWN)
  snDone = ffvDone = fbvDone = 0
  If Err.Number = 0 Then
    Do While Not WshScliExec.StdOut.AtEndOfStream And (fbvDone = 0 Or ffvDone = 0 Or snDone = 0)

      line = WshScliExec.StdOut.ReadLine

	If Len(line) > 0 And InStr(line,"Flash BIOS Version") And fbvDone = 0 Then
		colonIdx = InStr(line,":") + 2
		biosVer= Mid(line,colonIdx)
            fbvDone = 1
	End If

	If Len(line) > 0 And InStr(line,"Flash Firmware Version") And ffwDone = 0 Then
		colonIdx = InStr(line,":") + 2
		firmVer= Mid(line,colonIdx)
            ffvDone = 1
	End If

	If Len(line) > 0 And InStr(line,"Serial Number") And snDone = 0 Then
		colonIdx = InStr(line,":") + 2
		serialNum= Mid(line,colonIdx)
            snDone = 1
	End If

    Loop
  End If

End Function
'==============================
' PrintMetaInfo
'==============================
Function PrintMetaInfo()
' --Chassis--
  WScript.StdOut.WriteLine "Resource.Chassis.ClassName=Chassis"
  WScript.StdOut.WriteLine "Resource.Chassis.Defer=true"
  WScript.StdOut.WriteLine "Resource.Chassis.Keys=Tag"
  WScript.StdOut.WriteLine "Relationship.ChassisCS.ClassName=ComputerSystemContainsPhysicalElement"
  WScript.StdOut.WriteLine "Relationship.ChassisCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.ChassisCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.ChassisCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.ChassisCS.TargetClass=Chassis"
' --Processor--
  WScript.StdOut.WriteLine "Resource.Processor.ClassName=Processor"
  WScript.StdOut.WriteLine "Resource.Processor.Defer=true"
  WScript.StdOut.WriteLine "Resource.Processor.Keys=DeviceID"
  WScript.StdOut.WriteLine "Relationship.ProcessorCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.ProcessorCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.ProcessorCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.ProcessorCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.ProcessorCS.TargetClass=Processor"
' --PhysicalMemory--
  WScript.StdOut.WriteLine "Resource.PhysicalMemory.ClassName=PhysicalMemory"
  WScript.StdOut.WriteLine "Resource.PhysicalMemory.Defer=true"
  WScript.StdOut.WriteLine "Resource.PhysicalMemory.Keys=Tag"
  WScript.StdOut.WriteLine "Relationship.PhysicalMemoryCS.ClassName=ComputerSystemContainsPhysicalElement"
  WScript.StdOut.WriteLine "Relationship.PhysicalMemoryCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.PhysicalMemoryCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.PhysicalMemoryCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.PhysicalMemoryCS.TargetClass=PhysicalMemory"
' --DiskDrive--
  WScript.StdOut.WriteLine "Resource.DiskDrive.ClassName=DiskDrive"
  WScript.StdOut.WriteLine "Resource.DiskDrive.Defer=true"
  WScript.StdOut.WriteLine "Resource.DiskDrive.Keys=DeviceID"
  WScript.StdOut.WriteLine "Relationship.DiskDriveCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.DiskDriveCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.DiskDriveCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.DiskDriveCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.DiskDriveCS.TargetClass=DiskDrive"
' --LogicalVolume--
  WScript.StdOut.WriteLine "Resource.LogicalVolume.ClassName=LogicalVolume"
  WScript.StdOut.WriteLine "Resource.LogicalVolume.Defer=true"
  WScript.StdOut.WriteLine "Resource.LogicalVolume.Keys=DeviceID"
  WScript.StdOut.WriteLine "Relationship.LogicalVolumeCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.LogicalVolumeCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.LogicalVolumeCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.LogicalVolumeCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.LogicalVolumeCS.TargetClass=LogicalVolume"
' --FileSystem--
  WScript.StdOut.WriteLine "Resource.FileSystem.ClassName=FileSystem"
  WScript.StdOut.WriteLine "Resource.FileSystem.Defer=false"
  WScript.StdOut.WriteLine "Resource.FileSystem.Keys=Name"
  WScript.StdOut.WriteLine "Relationship.FileSystemCS.ClassName=ComputerSystemContainsFileSystem"
  WScript.StdOut.WriteLine "Relationship.FileSystemCS.Defer=false"
  WScript.StdOut.WriteLine "Relationship.FileSystemCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.FileSystemCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.FileSystemCS.TargetClass=FileSystem"
' --FibreChannel--
  WScript.StdOut.WriteLine "Resource.FCPort.ClassName=FCPort"
  WScript.StdOut.WriteLine "Resource.FCPort.Keys=DeviceID"
  WScript.StdOut.WriteLine "Resource.HBAAdapter.ClassName=PortController"
  WScript.StdOut.WriteLine "Resource.HBAAdapter.Keys=DeviceID"
  WScript.StdOut.WriteLine "Resource.HBACard.ClassName=Card"
  WScript.StdOut.WriteLine "Resource.HBACard.Keys=Tag"
  WScript.StdOut.WriteLine "Resource.HBASoftware.ClassName=SoftwareInstallation"
  WScript.StdOut.WriteLine "Resource.HBASoftware.Defer=false"
  WScript.StdOut.WriteLine "Resource.HBASoftware.Keys=Name,Version"
  WScript.StdOut.WriteLine "Resource.SCSIPort.ClassName=SCSIProtocolEndpoint"
  WScript.StdOut.WriteLine "Resource.SCSIPort.Keys=Name"
  WScript.StdOut.WriteLine "Relationship.FCPortCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.FCPortCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.FCPortCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.FCPortCS.TargetClass=FCPort"
  WScript.StdOut.WriteLine "Relationship.SCSIPortCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.SCSIPortCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.SCSIPortCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.SCSIPortCS.TargetClass=SCSIProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.HBAAdapterCS.ClassName=ComputerSystemContainsLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.HBAAdapterCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.HBAAdapterCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.HBAAdapterCS.TargetClass=PortController"
  WScript.StdOut.WriteLine "Relationship.HBAAdapterCS.TargetAlias=HBAAdapter"
  WScript.StdOut.WriteLine "Relationship.HBACardCS.ClassName=ComputerSystemContainsPhysicalElement"
  WScript.StdOut.WriteLine "Relationship.HBACardCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.HBACardCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.HBACardCS.TargetClass=Card"
  WScript.StdOut.WriteLine "Relationship.HBACardCS.TargetAlias=HBACard"
  WScript.StdOut.WriteLine "Relationship.HBASoftwareCS.ClassName=SoftwareResourceInstalledOnComputerSystem"
  WScript.StdOut.WriteLine "Relationship.HBASoftwareCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.HBASoftwareCS.SourceClass=SoftwareInstallation"
  WScript.StdOut.WriteLine "Relationship.HBASoftwareCS.SourceAlias=HBASoftware"
  WScript.StdOut.WriteLine "Relationship.HBASoftwareCS.TargetClass=Server"
  WScript.StdOut.WriteLine "Relationship.HBACardRealizes.ClassName=PhysicalElementRealizesLogicalDevice"
  WScript.StdOut.WriteLine "Relationship.HBACardRealizes.SourceClass=Card"
  WScript.StdOut.WriteLine "Relationship.HBACardRealizes.SourceAlias=HBACard"
  WScript.StdOut.WriteLine "Relationship.HBACardRealizes.TargetClass=PortController"
  WScript.StdOut.WriteLine "Relationship.HBACardRealizes.TargetAlias=HBAAdapter"
  WScript.StdOut.WriteLine "Relationship.HBAControlledBy.ClassName=LogicalDeviceControlledByController"
  WScript.StdOut.WriteLine "Relationship.HBAControlledBy.SourceClass=FCPort"
  WScript.StdOut.WriteLine "Relationship.HBAControlledBy.TargetClass=PortController"
  WScript.StdOut.WriteLine "Relationship.HBAControlledBy.TargetAlias=HBAAdapter"
  WScript.StdOut.WriteLine "Relationship.HBAAccessedVia.ClassName=LogicalDeviceAccessedViaServiceAccessPoint"
  WScript.StdOut.WriteLine "Relationship.HBAAccessedVia.SourceClass=FCPort"
  WScript.StdOut.WriteLine "Relationship.HBAAccessedVia.TargetClass=SCSIProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.HBAAccessedVia.TargetAlias=SCSIPort"
' --DNSSettings--
  WScript.StdOut.WriteLine "Resource.DNSSettings.ClassName=DNSSettings"
  WScript.StdOut.WriteLine "Resource.DNSSettings.Defer=true"
  WScript.StdOut.WriteLine "Resource.DNSSettings.Keys=SettingID"
  WScript.StdOut.WriteLine "Relationship.DNSSettingsCS.ClassName=ComputerSystemConfiguredUsingDNSSettings"
  WScript.StdOut.WriteLine "Relationship.DNSSettingsCS.Defer=true"
  WScript.StdOut.WriteLine "Relationship.DNSSettingsCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.DNSSettingsCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.DNSSettingsCS.TargetClass=DNSSettings"
' --NICs--
  WScript.StdOut.WriteLine "Resource.CSLANEndpoint.ClassName=LANEndpoint"
  WScript.StdOut.WriteLine "Resource.CSLANEndpoint.Keys=Name,HostType"
  WScript.StdOut.WriteLine "Resource.OSLANEndpoint.ClassName=LANEndpoint"
  WScript.StdOut.WriteLine "Resource.OSLANEndpoint.Keys=Name,HostType"
  WScript.StdOut.WriteLine "Resource.CSIPProtocolEndpoint.ClassName=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Resource.CSIPProtocolEndpoint.Keys=Name,HostType"
  WScript.StdOut.WriteLine "Resource.OSIPProtocolEndpoint.ClassName=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Resource.OSIPProtocolEndpoint.Keys=Name,HostType"
  WScript.StdOut.WriteLine "Relationship.LANEndpointCS.ClassName=SystemHostsProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.LANEndpointCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.LANEndpointCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.LANEndpointCS.TargetClass=LANEndpoint"
  WScript.StdOut.WriteLine "Relationship.LANEndpointCS.TargetAlias=CSLANEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointCS.ClassName=SystemHostsProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointCS.TargetClass=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointCS.TargetAlias=CSIPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.LANEndpointOS.ClassName=SystemHostsProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.LANEndpointOS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.LANEndpointOS.SourceClass=OperatingSystem"
  WScript.StdOut.WriteLine "Relationship.LANEndpointOS.TargetClass=LANEndpoint"
  WScript.StdOut.WriteLine "Relationship.LANEndpointOS.TargetAlias=OSLANEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointOS.ClassName=SystemHostsProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointOS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointOS.SourceClass=OperatingSystem"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointOS.TargetClass=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.IPProtocolEndpointOS.TargetAlias=OSIPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.CSBindsTo.ClassName=ProtocolEndpointBindsToProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.CSBindsTo.SourceClass=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.CSBindsTo.SourceAlias=CSIPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.CSBindsTo.TargetClass=LANEndpoint"
  WScript.StdOut.WriteLine "Relationship.CSBindsTo.TargetAlias=CSLANEndpoint"
  WScript.StdOut.WriteLine "Relationship.OSBindsTo.ClassName=ProtocolEndpointBindsToProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.OSBindsTo.SourceClass=IPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.OSBindsTo.SourceAlias=OSIPProtocolEndpoint"
  WScript.StdOut.WriteLine "Relationship.OSBindsTo.TargetClass=LANEndpoint"
  WScript.StdOut.WriteLine "Relationship.OSBindsTo.TargetAlias=OSLANEndpoint"
' --Location--
  WScript.StdOut.WriteLine "Resource.Location.ClassName=Location"
  WScript.StdOut.WriteLine "Resource.Location.Keys=Name"
  WScript.StdOut.WriteLine "Relationship.LocationCS.ClassName=SystemLocatedAtLocation"
  WScript.StdOut.WriteLine "Relationship.LocationCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.LocationCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.LocationCS.TargetClass=Location"
' --UserContact--
  WScript.StdOut.WriteLine "Resource.UserContact.ClassName=UserContact"
  WScript.StdOut.WriteLine "Resource.UserContact.Keys=Name"
  WScript.StdOut.WriteLine "Relationship.UserContactCS.ClassName=SystemExposesUserContact"
  WScript.StdOut.WriteLine "Relationship.UserContactCS.Implicit=true"
  WScript.StdOut.WriteLine "Relationship.UserContactCS.SourceClass=Server"
  WScript.StdOut.WriteLine "Relationship.UserContactCS.TargetClass=UserContact"
End Function
